/**
  ******************************************************************************
  * @file           : main.h
  * @brief          : Header for main.c file.
  *                   This file contains the common defines of the application.
  ******************************************************************************
  */

// Define to prevent recursive inclusion -------------------------------------
#ifndef __MAIN_H__
#define __MAIN_H__

// Includes ------------------------------------------------------------------
#include "stm32f4xx_hal.h"
#include "stm32f4xx_ll_usart.h"
#include "stm32f4xx_ll_rcc.h"
#include "stm32f4xx_ll_bus.h"
#include "stm32f4xx_ll_cortex.h"
#include "stm32f4xx_ll_system.h"
#include "stm32f4xx_ll_utils.h"
#include "stm32f4xx_ll_pwr.h"
#include "stm32f4xx_ll_gpio.h"
#include "stm32f4xx_ll_dma.h"

#include "stm32f4xx_ll_exti.h"

/* USER CODE BEGIN Includes */

/* USER CODE END Includes */

// Private define ------------------------------------------------------------

#define LED3_Pin GPIO_PIN_13
#define LED3_GPIO_Port GPIOC
#define LED2_Pin GPIO_PIN_14
#define LED2_GPIO_Port GPIOC
#define LED1_Pin GPIO_PIN_15
#define LED1_GPIO_Port GPIOC
#define AnIn1_Pin GPIO_PIN_0
#define AnIn1_GPIO_Port GPIOC
#define AnIn2_Pin GPIO_PIN_1
#define AnIn2_GPIO_Port GPIOC
#define LCD_DB4_Pin GPIO_PIN_0
#define LCD_DB4_GPIO_Port GPIOA
#define LCD_DB5_Pin GPIO_PIN_1
#define LCD_DB5_GPIO_Port GPIOA
#define LCD_DB6_Pin GPIO_PIN_2
#define LCD_DB6_GPIO_Port GPIOA
#define LCD_DB7_Pin GPIO_PIN_3
#define LCD_DB7_GPIO_Port GPIOA
#define LCD_RS_Pin GPIO_PIN_5
#define LCD_RS_GPIO_Port GPIOA
#define LCD_E_Pin GPIO_PIN_6
#define LCD_E_GPIO_Port GPIOA
#define AnIn3_Pin GPIO_PIN_4
#define AnIn3_GPIO_Port GPIOC
#define AnIn4_Pin GPIO_PIN_5
#define AnIn4_GPIO_Port GPIOC
#define Srf_Int_Pin GPIO_PIN_0
#define Srf_Int_GPIO_Port GPIOB
#define Srf_Int_EXTI_IRQn EXTI0_IRQn
#define Boot1_Pin GPIO_PIN_2
#define Boot1_GPIO_Port GPIOB
#define TxMIDI_Pin GPIO_PIN_10
#define TxMIDI_GPIO_Port GPIOB
#define RxMIDI_Pin GPIO_PIN_11
#define RxMIDI_GPIO_Port GPIOB
#define USB_DM_Pin GPIO_PIN_14
#define USB_DM_GPIO_Port GPIOB
#define USB_DP_Pin GPIO_PIN_15
#define USB_DP_GPIO_Port GPIOB
#define SoftMute1_Pin GPIO_PIN_8
#define SoftMute1_GPIO_Port GPIOC
#define SoftMute2_Pin GPIO_PIN_9
#define SoftMute2_GPIO_Port GPIOC
#define USB_D__Pin GPIO_PIN_11
#define USB_D__GPIO_Port GPIOA
#define USB_D_A12_Pin GPIO_PIN_12
#define USB_D_A12_GPIO_Port GPIOA
#define SWDIO_Pin GPIO_PIN_13
#define SWDIO_GPIO_Port GPIOA
#define SWCLK_Pin GPIO_PIN_14
#define SWCLK_GPIO_Port GPIOA
#define LED4_Pin GPIO_PIN_11
#define LED4_GPIO_Port GPIOC
#define Marker_Pin GPIO_PIN_2
#define Marker_GPIO_Port GPIOD
#define TxAUX_Pin GPIO_PIN_6
#define TxAUX_GPIO_Port GPIOB
#define RxAUX_Pin GPIO_PIN_7
#define RxAUX_GPIO_Port GPIOB
#define SCL_Pin GPIO_PIN_8
#define SCL_GPIO_Port GPIOB
#define SDA_Pin GPIO_PIN_9
#define SDA_GPIO_Port GPIOB

// ########################## Assert Selection ############################## 
/**
  * @brief Uncomment the line below to expanse the "assert_param" macro in the 
  *        HAL drivers code
  */
/* #define USE_FULL_ASSERT    1U */

/* USER CODE BEGIN Private defines */
typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned long u32;

typedef enum
{ false, true } bool;

#define enableInterrupts()   __set_PRIMASK(0);
#define disableInterrupts()  __set_PRIMASK(1);
#define pinSet(Port,Pin)     Port->BSRR = (Pin)
#define pinReset(Port,Pin)   Port->BSRR = (uint32_t)(Pin) << 16U
#define pinToggle(Port,Pin)  Port->ODR ^= (Pin)
#define pinRead(Port,Pin)    !!(Port->IDR & (Pin))

#define AUDIO_BUFFER_SIZE 0x400

typedef enum
{
  SRF_NOP, GET_FW_VERSION, READ_SRF_CLEAR, ENC5_DATA, ENC5_VOLUME,
  SET_MAX_VOLUMES_CH1, SET_MAX_VOLUMES_CH2, SET_VOLUME_STEPS,
} SrfCommands;

typedef struct __attribute__ ( ( packed ) )
{
  int           enc1:4, enc2:4, enc3:4, enc4:4, enc5:4;
  int           swe1:1, swe2:1, swe3:1, swe4:1, swe5:1;
  int           sw1:1, sw2:1, sw3:1, sw4:1, sw5:1, sw6:1, sw7:1;
} Surface;

typedef struct __attribute__ ( ( packed ) )
{
  unsigned int  encoders:20;
  unsigned int  switches:12;
} Surface_bits;

typedef struct __attribute__ ( ( packed ) )
{
  unsigned int  day:5;
  unsigned int  month:4;
  unsigned int  year:11;
  unsigned int  channels:2;
  unsigned int  amplifier:1;
} FwVersion;

typedef struct __attribute__ ( ( packed ) )
{
  u16           command;
  u16           param;
} SrfCommand;

typedef struct __attribute__ ( ( packed ) )
{
  u8            L1, R1, L2, R2;
} Volume;

/* USER CODE END Private defines */

#ifdef __cplusplus
 extern "C" {
#endif
void _Error_Handler(char *, int);

#define Error_Handler() _Error_Handler(__FILE__, __LINE__)
#ifdef __cplusplus
}
#endif

#endif /* __MAIN_H__ */

