#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#define SAMPLES 4096

char         *octave[12] = { "A ", "Bb", "B ", "C ",
  "C#", "D ", "Eb", "E ", "F ", "F#", "G ", "G#"
};

int
main( int argc, char *argv[] )
{
  int           i, smp;
  FILE         *nt;

  if( ( nt = fopen( "notes.h", "wt" ) ) == NULL )
    exit( 0 );
  fprintf( nt, "enum EventType {\n" );
  fprintf( nt, "    MIDI_NOTE_OFF = 0x80,\n" );
  fprintf( nt, "    MIDI_NOTE_ON = 0x90,\n" );
  fprintf( nt, "    MIDI_POLY_AFTER_TOUCH = 0xA0,\n" );
  fprintf( nt, "    MIDI_CONTROL_CHANGE = 0xb0,\n" );
  fprintf( nt, "    MIDI_PROGRAM_CHANGE =0xc0,\n" );
  fprintf( nt, "    MIDI_AFTER_TOUCH = 0xd0,\n" );
  fprintf( nt, "    MIDI_PITCH_BEND = 0xe0,\n" );
  fprintf( nt, "    MIDI_SYSTEM_COMMON = 0xf0,\n" );
  fprintf( nt, "    MIDI_SYSEX = 0xf0,\n" );
  fprintf( nt, "    MIDI_SONG_POSITION = 0xf2,\n" );
  fprintf( nt, "    MIDI_START = 0xfa,\n" );
  fprintf( nt, "    MIDI_CLOCK = 0xf8,\n" );
  fprintf( nt, "    MIDI_CONTINUE = 0xfb,\n" );
  fprintf( nt, "    MIDI_STOP = 0xfc,\n" );
  fprintf( nt, "    MIDI_SYSEX_END = 0xf7\n" );
  fprintf( nt, "};\n\n" );
  fprintf( nt, "enum EventState {\n" );
  fprintf( nt, "    MIDI_EVENT_WAITING = 0,\n" );
  fprintf( nt, "    MIDI_EVENT_IN_PROGRESS ,\n" );
  fprintf( nt, "    MIDI_EVENT_SYSEX ,\n" );
  fprintf( nt, "    MIDI_EVENT_COMPLETE\n" );
  fprintf( nt, "};\n\n" );
  fprintf( nt, "#define SINE_NR_SAMPLES %d\n", SAMPLES );
  fprintf( nt, "#define MAX_NOTES 88\n\n" );
  fprintf( nt,
      "typedef struct\n{\n\tconst char *note;\n\tconst float freq;\n} Note;\n\n" );
  fprintf( nt, "const Note notes[MAX_NOTES] = {\n" );
  for( i = -48; i < 40; i++ )
  {
    fprintf( nt, "{\"%s%d\", %.5f}, ", 
	octave[( i + 48 ) % 12], ( i + 48 + 9 ) / 12, 440. * pow( 2.,
	i / 12. ) );
    if( ( ( i + 48 ) & 3 ) == 3 )
      fprintf( nt, "\n" );
  }
  fprintf( nt, "};\n\n" );
  fprintf( nt, "const int sine_tbl[SINE_NR_SAMPLES] = {\n" );
  for( i = 0; i < SAMPLES; i++ )
  {
    smp = sin( M_PI * 2 * i / SAMPLES ) * 0x72000000;
    fprintf( nt, "0x%x, ", smp );
    if( ( i & 3 ) == 3 )
      fprintf( nt, "\n" );
  }
  fprintf( nt, "};\n\n" );
  fclose( nt );
  printf( "Done\n" );
  return 0;
}
