/**
  ******************************************************************************
  * @file	    : main.c
  * @brief	    : Main program body
  ******************************************************************************
  */
// Includes ------------------------------------------------------------------
#include "main.h"
#include "stm32f4xx_hal.h"
#include "usb_device.h"

/* USER CODE BEGIN Includes */
#include "notes.h"
#include "LiquidCrystal.h"
/* USER CODE END Includes */

// Private variables ---------------------------------------------------------
ADC_HandleTypeDef hadc1;

I2C_HandleTypeDef hi2c1;

I2S_HandleTypeDef hi2s2;
I2S_HandleTypeDef hi2s3;
DMA_HandleTypeDef hdma_spi2_tx;
DMA_HandleTypeDef hdma_spi3_tx;

SPI_HandleTypeDef hspi1;
DMA_HandleTypeDef hdma_spi1_rx;
DMA_HandleTypeDef hdma_spi1_tx;

HCD_HandleTypeDef hhcd_USB_OTG_HS;

/* USER CODE BEGIN PV */
// Private variables ---------------------------------------------------------
volatile union
{
  Surface       sf;
  Surface_bits  sfb;
} spi_rx_data;
SrfCommand    spi_cmd;
FwVersion     fw_version;

int           buffer[2][AUDIO_BUFFER_SIZE];
int          *boffset[2];
int           Encoders[4] = { 27, 31, 34, 38 };	// Cmaj7 - C3, E3, G3, B3

bool          note_demo;
struct
{
  int           key, amplitude;
  float         step, sample;
} Keys[4];
/* USER CODE END PV */

// Private function prototypes -----------------------------------------------
void          SystemClock_Config( void );
static void   MX_GPIO_Init( void );
static void   MX_DMA_Init( void );
static void   MX_SPI1_Init( void );
static void   MX_I2S2_Init( void );
static void   MX_I2S3_Init( void );
static void   MX_I2C1_Init( void );
static void   MX_ADC1_Init( void );
static void   MX_USART1_UART_Init( void );
static void   MX_USART3_UART_Init( void );

/* USER CODE BEGIN PFP */
// Private function prototypes -----------------------------------------------

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */
void 
HAL_SPI_ErrorCallback(SPI_HandleTypeDef * hspi)
{
  HAL_SPI_Abort( hspi );
  pinReset( LED1_GPIO_Port, LED1_Pin );
  pinReset( LED4_GPIO_Port, LED4_Pin );
}

void
HAL_SPI_TxRxCpltCallback( SPI_HandleTypeDef * hspi )
{
  u32           encoders = spi_rx_data.sfb.encoders;
  u32           switches = spi_rx_data.sfb.switches;
  const Note   *n;
  int           i, delta;
  if( !note_demo )
    return;
  for( i = 0; i < 4; i++ )
  {
    if( switches & ( 1 << i ) )
    {
      Keys[i].amplitude = Keys[i].amplitude ? 0 : 127;
      LCD_char( 19, i, Keys[i].amplitude ? 7 : ' ' );
    }
    delta = encoders & 0xf;
    if( encoders & 8 )		// extend sign
      delta |= -8;
    encoders >>= 4;
    if( delta == 0 )
      continue;
    if( Encoders[i] + delta >= 0 && Encoders[i] + delta < MAX_NOTES )
      Encoders[i] += delta;
    else
      continue;
    n = &notes[Encoders[i]];
    LCD_print( 0, i, "Ch%d%c %3s %7.2fHz",
	1 + i / 2, ( i & 1 ) ? 'R' : 'L', n->note, n->freq );
    Keys[i].step = n->freq * SINE_NR_SAMPLES / 48e3;
  }
}

void
HAL_GPIO_EXTI_Callback( uint16_t GPIO_Pin )
{
  u8           *receive;
  if( GPIO_Pin != GPIO_PIN_0 )
    return;
  // surface interrupt
  switch ( spi_cmd.command )
  {
    case GET_FW_VERSION:
      spi_cmd.command = SRF_NOP;
      receive = ( u8 * ) & fw_version;
      break;
    default:
      spi_cmd.command = READ_SRF_CLEAR;
      receive = ( u8 * ) & spi_rx_data;
      break;
  }
  while( HAL_SPI_TransmitReceive_DMA( &hspi1, ( u8 * ) & spi_cmd, receive,
      2 ) != HAL_OK )
    HAL_SPI_Abort( &hspi1 );
}

void 
resync_spi(void)
{
 int ret;
        spi_cmd.command = SRF_NOP;
spi_cmd.param = SRF_NOP;
        ret = HAL_SPI_Transmit( &hspi1, ( u8 * ) & spi_cmd, 2, 5 );
}

void
surface_info( void )
{
  u32 timestamp, delta;
  LCD_print( 2, 0, "Surface Control" );
  timestamp = HAL_GetTick();
  do
  {
    spi_cmd.command = GET_FW_VERSION;
    while( HAL_SPI_TransmitReceive_DMA( &hspi1, ( u8 * ) & spi_cmd,
	( u8 * ) & spi_rx_data, 2 ) != HAL_OK )
      HAL_SPI_Abort( &hspi1 );
    // wait for the data ready from surface CPU
    do
    {
      delta = HAL_GetTick() - timestamp;
      if( delta > 3 )
      {
resync_spi();
        break;
      }
    }
    while( !pinRead( Srf_Int_GPIO_Port, Srf_Int_Pin ) );
    if( delta > 3 )
      continue;
    // wait for the data transfer to complete
    do
    {
      delta = HAL_GetTick() - timestamp;
      if( delta > 3 )
      {
      resync_spi();
        break;
      }
    }
    while( pinRead( Srf_Int_GPIO_Port, Srf_Int_Pin ) );
    if( delta > 3 )
      continue;
  }
  while( fw_version.year < 2000 || fw_version.year > 2100 );
  LCD_print( 1, 1, "version %d-%d-%d", fw_version.day, fw_version.month,
      fw_version.year );
  LCD_print( 3, 2, "hardware: %dch ", fw_version.channels );
  if( fw_version.amplifier )
    LCD_print( 0, 3, "with headphones amp" );
  HAL_Delay( 1000 );
  LiquidCrystal_clear(  );
}

void
create_dot( void )
{
  static u8     dot[8] = { 0, 4, 0xe, 0x1f, 0x1f, 0xe, 4, 0 };
  LiquidCrystal_createChar( 7, dot );
}

void
test_all_switches( void )
{
  int           i, sw, mem_sw = 0;
  return;
  LCD_print( 1, 0, "Please press each" );
  LCD_print( 10, 1, "1234567" );
  LCD_print( 1, 2, "switch:" );
  LCD_print( 1, 3, "encoder:" );
  do
  {
    sw = spi_rx_data.sfb.switches;
    for( i = 0; i < 7; i++ )
      LCD_char( 10 + i, 2, sw & ( 1 << ( i + 5 ) ) ? 
          0xff : mem_sw & ( 1 << ( i + 5 ) ) ? 7 : ' ' );
    for( i = 0; i < 5; i++ )
      LCD_char( 10 + i, 3,
	  sw & ( 1 << i ) ? 0xff : mem_sw & ( 1 << i ) ? 7 : ' ' );
    HAL_Delay( 20 );
    mem_sw |= sw;
  }
  while( sw != 0 || mem_sw != ( 1 << 12 ) - 1 );
  HAL_Delay( 1000 );
  LiquidCrystal_clear(  );
}

void
test_all_encoders( void )
{
  int           enc[5] = { 0, 0, 0, 0, 0 };
  spi_cmd.command = ENC5_DATA;
  HAL_SPI_TransmitReceive( &hspi1, ( u8 * ) & spi_cmd, ( u8 * ) & spi_rx_data,
      2, 30 );
  LCD_print( 0, 0, "Rotate each encoder" );
  LCD_print( 0, 1, "Enc1 Enc2 Enc3 Enc4" );
  LCD_print( 0, 3, "Enc5" );
  LCD_print( 13, 3, "Cont ~" );
  spi_rx_data.sfb.switches = 0;
  spi_rx_data.sfb.encoders = 0;
  do
  {
    enc[0] += spi_rx_data.sf.enc1;
    enc[1] += spi_rx_data.sf.enc2;
    enc[2] += spi_rx_data.sf.enc3;
    enc[3] += spi_rx_data.sf.enc4;
    enc[4] += spi_rx_data.sf.enc5;
    spi_rx_data.sfb.encoders = 0;
    LCD_print( 0, 2, "%+4d", enc[0] );
    LCD_print( 5, 2, "%+4d", enc[1] );
    LCD_print( 10, 2, "%+4d", enc[2] );
    LCD_print( 15, 2, "%+4d", enc[3] );
    LCD_print( 5, 3, "%+4d", enc[4] );
    HAL_Delay( 40 );
  }
  while( spi_rx_data.sf.sw2 == 0 );
  while( spi_rx_data.sf.sw2 != 0 ) ;
  spi_rx_data.sfb.switches = 0;
  LiquidCrystal_clear(  );
  spi_cmd.command = ENC5_VOLUME;
  HAL_SPI_TransmitReceive( &hspi1, ( u8 * ) & spi_cmd, ( u8 * ) & spi_rx_data,
      2, 30 );
}

void
display_notes( void )
{
  const Note   *n;
  int           i;
  LCD_print( 0, 0, "Use the 4 encoders" );
  LCD_print( 0, 1, "to select/activate" );
  LCD_print( 0, 2, "the note for each" );
  LCD_print( 0, 3, "channel. Continue ~" );
  while( spi_rx_data.sf.sw2 == 0 ) ;
  while( spi_rx_data.sf.sw2 != 0 ) ;
  LiquidCrystal_clear(  );
  note_demo = true;
  for( i = 0; i < 4; i++ )
  {
    n = &notes[Encoders[i]];
    LCD_print( 0, i, "Ch%d%c %3s %7.2fHz",
	1 + i / 2, ( i & 1 ) ? 'R' : 'L', n->note, n->freq );
    Keys[i].step = n->freq * SINE_NR_SAMPLES / 48e3;
  }
}

void
fill_half_buffer( int nr )
{
  int           i, idx, n, smp;

  for( i = 0; i < AUDIO_BUFFER_SIZE / 2; i++ )
  {
    idx = ( nr << 1 ) | ( i & 1 );
    if( Keys[idx].amplitude )
    {
      n = Keys[idx].sample + 0.5;
      if( n >= SINE_NR_SAMPLES )
      {
	n -= SINE_NR_SAMPLES;
	Keys[idx].sample -= SINE_NR_SAMPLES;
      }
      Keys[idx].sample += Keys[idx].step;
      if( Keys[idx].amplitude == 127 )
        smp = sine_tbl[n];
      else
        smp = ( sine_tbl[n] >> 7 ) * Keys[idx].amplitude;
      smp = ( smp & 0xffff0000 ) >> 16 | ( smp & 0xffff ) << 16;
      *boffset[nr]++ = smp;
    }
    else
      *boffset[nr]++ = 0;
  }
  if( boffset[nr] == buffer[nr] + AUDIO_BUFFER_SIZE )
    boffset[nr] = buffer[nr];
}

void
HAL_I2S_TxHalfCpltCallback( I2S_HandleTypeDef * hi2s )
{
  fill_half_buffer( hi2s == &hi2s3 );
}

void
HAL_I2S_TxCpltCallback( I2S_HandleTypeDef * hi2s )
{
  fill_half_buffer( hi2s == &hi2s3 );
}

static inline void
enable_irq( USART_TypeDef * usart )
{
  LL_USART_EnableIT_RXNE( usart );
  LL_USART_EnableIT_ERROR( usart );
}

u8            MIDI_buf[10];
enum EventType eventType;
int           MIDI_bytes, MIDI_idx;
int           MIDI_channel;

void
noteOn( int note, int velocity )
{
  const Note   *n;
  int           i;

  if( note < 0 || note > MAX_NOTES )
    return;
  for( i = 0; i < 4; i++ )
  {
    if( Keys[i].amplitude )
      continue;
    Keys[i].key = note;
    n = &notes[note];
    LCD_print( 5, i, "%3s %7.2fHz", n->note, n->freq );
    Keys[i].step = n->freq * SINE_NR_SAMPLES / 48e3;
    Keys[i].amplitude = velocity;
    LCD_char( 19, i, 7 );
    break;
  }
}

void
noteOff( int note )
{
  int           i;

  if( note < 0 || note > MAX_NOTES )
    return;
  for( i = 0; i < 4; i++ )
  {
    if( Keys[i].key == note )
    {
      Keys[i].amplitude = 0;
      LCD_char( 19, i, ' ' );
    }
  }
}

void
MIDI_Received( void )
{
  switch ( eventType )
  {
    case MIDI_NOTE_OFF:
      noteOff( MIDI_buf[1] - 21 );
      break;
    case MIDI_NOTE_ON:
      // Some keyboards send note-off this way
      if( MIDI_buf[2] == 0 )
	noteOff( MIDI_buf[1] - 21 );
      else
	noteOn( MIDI_buf[1] - 21, MIDI_buf[2] );
      break;
    case MIDI_CONTROL_CHANGE:
      // controlChange( timbres[tk], midiEvent );
      break;
    case MIDI_POLY_AFTER_TOUCH:
      break;
    case MIDI_AFTER_TOUCH:
      break;
    case MIDI_PITCH_BEND:
      break;
    case MIDI_PROGRAM_CHANGE:
      break;
    case MIDI_SONG_POSITION:
      break;
  }
}

void
MIDI_Decoder( u8 byte )
{
  eventType = byte & 0xf0;
  MIDI_channel = byte & 0x0f;
  switch ( eventType )
  {
    case MIDI_NOTE_OFF:
    case MIDI_NOTE_ON:
    case MIDI_CONTROL_CHANGE:
    case MIDI_PITCH_BEND:
    case MIDI_POLY_AFTER_TOUCH:
      MIDI_bytes = 3;
      break;
    case MIDI_AFTER_TOUCH:
    case MIDI_PROGRAM_CHANGE:
      MIDI_bytes = 2;
      break;
    case MIDI_SYSTEM_COMMON:
      switch ( byte )
      {
	case MIDI_SYSEX:
	  break;
      }
      break;
  }
}

void
Serial_IRQHandler( USART_TypeDef * usart )
{
  int           error = 0;
  u8            byte;
  if( LL_USART_IsActiveFlag_FE( usart ) )
  {
    LL_USART_ClearFlag_FE( usart );
    error = 1;
  }
  if( LL_USART_IsActiveFlag_NE( usart ) )
  {
    LL_USART_ClearFlag_NE( usart );
    error = 1;
  }
  if( LL_USART_IsActiveFlag_ORE( usart ) )
  {
    LL_USART_ClearFlag_ORE( usart );
    error = 1;
  }
  if( LL_USART_IsActiveFlag_RXNE( usart ) )
  {
    byte = LL_USART_ReceiveData8( usart );
    if( byte & 0x80 )
    {
      MIDI_idx = 0;
      MIDI_Decoder( byte );
    }
    MIDI_buf[MIDI_idx++] = byte;
    if( MIDI_idx == MIDI_bytes )
    {
      MIDI_Received(  );
      MIDI_idx = 0;
    }
  }
  enable_irq( usart );
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  *
  * @retval None
  */
int
main( void )
{
  /* USER CODE BEGIN 1 */
  GPIO_InitTypeDef GPIO_InitStruct;
  /* USER CODE END 1 */

  /* MCU Configuration---------------------------------------------------------- */

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init(  );

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config(  );

  /* USER CODE BEGIN SysInit */
  // first enable the marker pin as a input pull-up
  __HAL_RCC_GPIOD_CLK_ENABLE(  );
  GPIO_InitStruct.Pin = Marker_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init( Marker_GPIO_Port, &GPIO_InitStruct );
  // keep the processor in this loop if it's not the SMD version of the board
  do
  {
    HAL_Delay( 5 );
  }
  while( pinRead( Marker_GPIO_Port, Marker_Pin ) );
  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init(  );
  MX_DMA_Init(  );
  MX_SPI1_Init(  );
  MX_I2S2_Init(  );
  MX_I2S3_Init(  );
  MX_I2C1_Init(  );
  MX_USB_DEVICE_Init(  );
  MX_ADC1_Init(  );
  MX_USART1_UART_Init(  );
  MX_USART3_UART_Init(  );
  /* USER CODE BEGIN 2 */
  pinSet( LED1_GPIO_Port, LED1_Pin );
  pinSet( LED2_GPIO_Port, LED2_Pin );
  pinSet( LED3_GPIO_Port, LED3_Pin );
  pinSet( LED4_GPIO_Port, LED4_Pin );
  LiquidCrystal_begin( 20, 4 );
  create_dot(  );
  surface_info(  );
  test_all_switches(  );
  test_all_encoders(  );
  display_notes(  );
  pinSet( SoftMute1_GPIO_Port, SoftMute1_Pin );
  pinSet( SoftMute2_GPIO_Port, SoftMute2_Pin );
  enable_irq( USART1 );
  enable_irq( USART3 );

  // init circular buffer pointers
  boffset[0] = buffer[0];
  fill_half_buffer( 0 );
  fill_half_buffer( 0 );
  HAL_I2S_Transmit_DMA( &hi2s2, ( u16 * ) buffer[0], AUDIO_BUFFER_SIZE );
  boffset[1] = buffer[1];
  fill_half_buffer( 1 );
  fill_half_buffer( 1 );
  HAL_I2S_Transmit_DMA( &hi2s3, ( u16 * ) buffer[1], AUDIO_BUFFER_SIZE );
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while( 1 )
  {
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
    // sleep all the time - code is interrupt driven
    HAL_PWR_EnterSLEEPMode( PWR_MAINREGULATOR_ON, PWR_SLEEPENTRY_WFI );
  }
  /* USER CODE END 3 */

}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void
SystemClock_Config( void )
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInitStruct;

    /**Configure the main internal regulator output voltage
    */
  __HAL_RCC_PWR_CLK_ENABLE(  );

  __HAL_PWR_VOLTAGESCALING_CONFIG( PWR_REGULATOR_VOLTAGE_SCALE1 );

    /**Initializes the CPU, AHB and APB busses clocks
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 12;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  if( HAL_RCC_OscConfig( &RCC_OscInitStruct ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

    /**Initializes the CPU, AHB and APB busses clocks
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_SYSCLK
      | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV4;

  if( HAL_RCC_ClockConfig( &RCC_ClkInitStruct, FLASH_LATENCY_5 ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

  PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_I2S;
  PeriphClkInitStruct.PLLI2S.PLLI2SN = 384;
  PeriphClkInitStruct.PLLI2S.PLLI2SR = 5;
  if( HAL_RCCEx_PeriphCLKConfig( &PeriphClkInitStruct ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

    /**Configure the Systick interrupt time
    */
  HAL_SYSTICK_Config( HAL_RCC_GetHCLKFreq(  ) / 1000 );

    /**Configure the Systick
    */
  HAL_SYSTICK_CLKSourceConfig( SYSTICK_CLKSOURCE_HCLK );

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority( SysTick_IRQn, 5, 0 );
}

// ADC1 init function
static void
MX_ADC1_Init( void )
{

  ADC_ChannelConfTypeDef sConfig;

    /**Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion)
    */
  hadc1.Instance = ADC1;
  hadc1.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV2;
  hadc1.Init.Resolution = ADC_RESOLUTION_12B;
  hadc1.Init.ScanConvMode = DISABLE;
  hadc1.Init.ContinuousConvMode = DISABLE;
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc1.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 4;
  hadc1.Init.DMAContinuousRequests = DISABLE;
  hadc1.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  if( HAL_ADC_Init( &hadc1 ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

    /**Configure for the selected ADC regular channel its corresponding rank in the sequencer and its sample time.
    */
  sConfig.Channel = ADC_CHANNEL_10;
  sConfig.Rank = 1;
  sConfig.SamplingTime = ADC_SAMPLETIME_3CYCLES;
  if( HAL_ADC_ConfigChannel( &hadc1, &sConfig ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

    /**Configure for the selected ADC regular channel its corresponding rank in the sequencer and its sample time.
    */
  sConfig.Channel = ADC_CHANNEL_11;
  sConfig.Rank = 2;
  if( HAL_ADC_ConfigChannel( &hadc1, &sConfig ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

    /**Configure for the selected ADC regular channel its corresponding rank in the sequencer and its sample time.
    */
  sConfig.Channel = ADC_CHANNEL_14;
  sConfig.Rank = 3;
  if( HAL_ADC_ConfigChannel( &hadc1, &sConfig ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

    /**Configure for the selected ADC regular channel its corresponding rank in the sequencer and its sample time.
    */
  sConfig.Channel = ADC_CHANNEL_15;
  sConfig.Rank = 4;
  if( HAL_ADC_ConfigChannel( &hadc1, &sConfig ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

}

// I2C1 init function
static void
MX_I2C1_Init( void )
{

  hi2c1.Instance = I2C1;
  hi2c1.Init.ClockSpeed = 400000;
  hi2c1.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if( HAL_I2C_Init( &hi2c1 ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

}

// I2S2 init function
static void
MX_I2S2_Init( void )
{

  hi2s2.Instance = SPI2;
  hi2s2.Init.Mode = I2S_MODE_MASTER_TX;
  hi2s2.Init.Standard = I2S_STANDARD_PHILIPS;
  hi2s2.Init.DataFormat = I2S_DATAFORMAT_24B;
  hi2s2.Init.MCLKOutput = I2S_MCLKOUTPUT_DISABLE;
  hi2s2.Init.AudioFreq = I2S_AUDIOFREQ_48K;
  hi2s2.Init.CPOL = I2S_CPOL_LOW;
  hi2s2.Init.ClockSource = I2S_CLOCK_PLL;
  hi2s2.Init.FullDuplexMode = I2S_FULLDUPLEXMODE_DISABLE;
  if( HAL_I2S_Init( &hi2s2 ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

}

// I2S3 init function
static void
MX_I2S3_Init( void )
{

  hi2s3.Instance = SPI3;
  hi2s3.Init.Mode = I2S_MODE_MASTER_TX;
  hi2s3.Init.Standard = I2S_STANDARD_PHILIPS;
  hi2s3.Init.DataFormat = I2S_DATAFORMAT_24B;
  hi2s3.Init.MCLKOutput = I2S_MCLKOUTPUT_DISABLE;
  hi2s3.Init.AudioFreq = I2S_AUDIOFREQ_48K;
  hi2s3.Init.CPOL = I2S_CPOL_LOW;
  hi2s3.Init.ClockSource = I2S_CLOCK_PLL;
  hi2s3.Init.FullDuplexMode = I2S_FULLDUPLEXMODE_DISABLE;
  if( HAL_I2S_Init( &hi2s3 ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

}

// SPI1 init function
static void
MX_SPI1_Init( void )
{

  /* SPI1 parameter configuration */
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_16BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_HARD_OUTPUT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_32;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if( HAL_SPI_Init( &hspi1 ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

}

// USART1 init function
static void
MX_USART1_UART_Init( void )
{

  LL_USART_InitTypeDef USART_InitStruct;

  LL_GPIO_InitTypeDef GPIO_InitStruct;

  /* Peripheral clock enable */
  LL_APB2_GRP1_EnableClock( LL_APB2_GRP1_PERIPH_USART1 );

  /**USART1 GPIO Configuration
  PB6	------> USART1_TX
  PB7	------> USART1_RX
  */
  GPIO_InitStruct.Pin = LL_GPIO_PIN_6 | LL_GPIO_PIN_7;
  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
  GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
  GPIO_InitStruct.Alternate = LL_GPIO_AF_7;
  LL_GPIO_Init( GPIOB, &GPIO_InitStruct );

  /* USART1 interrupt Init */
  NVIC_SetPriority( USART1_IRQn,
      NVIC_EncodePriority( NVIC_GetPriorityGrouping(  ), 7, 0 ) );
  NVIC_EnableIRQ( USART1_IRQn );

  USART_InitStruct.BaudRate = 31250;
  USART_InitStruct.DataWidth = LL_USART_DATAWIDTH_8B;
  USART_InitStruct.StopBits = LL_USART_STOPBITS_1;
  USART_InitStruct.Parity = LL_USART_PARITY_NONE;
  USART_InitStruct.TransferDirection = LL_USART_DIRECTION_TX_RX;
  USART_InitStruct.HardwareFlowControl = LL_USART_HWCONTROL_NONE;
  USART_InitStruct.OverSampling = LL_USART_OVERSAMPLING_16;
  LL_USART_Init( USART1, &USART_InitStruct );

  LL_USART_ConfigAsyncMode( USART1 );

  LL_USART_Enable( USART1 );

}

// USART3 init function
static void
MX_USART3_UART_Init( void )
{

  LL_USART_InitTypeDef USART_InitStruct;

  LL_GPIO_InitTypeDef GPIO_InitStruct;

  /* Peripheral clock enable */
  LL_APB1_GRP1_EnableClock( LL_APB1_GRP1_PERIPH_USART3 );

  /**USART3 GPIO Configuration
  PB10	 ------> USART3_TX
  PB11	 ------> USART3_RX
  */
  GPIO_InitStruct.Pin = LL_GPIO_PIN_10 | LL_GPIO_PIN_11;
  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
  GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
  GPIO_InitStruct.Alternate = LL_GPIO_AF_7;
  LL_GPIO_Init( GPIOB, &GPIO_InitStruct );

  /* USART3 interrupt Init */
  NVIC_SetPriority( USART3_IRQn,
      NVIC_EncodePriority( NVIC_GetPriorityGrouping(  ), 7, 0 ) );
  NVIC_EnableIRQ( USART3_IRQn );

  USART_InitStruct.BaudRate = 31250;
  USART_InitStruct.DataWidth = LL_USART_DATAWIDTH_8B;
  USART_InitStruct.StopBits = LL_USART_STOPBITS_1;
  USART_InitStruct.Parity = LL_USART_PARITY_NONE;
  USART_InitStruct.TransferDirection = LL_USART_DIRECTION_TX_RX;
  USART_InitStruct.HardwareFlowControl = LL_USART_HWCONTROL_NONE;
  USART_InitStruct.OverSampling = LL_USART_OVERSAMPLING_16;
  LL_USART_Init( USART3, &USART_InitStruct );

  LL_USART_ConfigAsyncMode( USART3 );

  LL_USART_Enable( USART3 );

}

// USB_OTG_HS init function
void
MX_USB_OTG_HS_HCD_Init( void )
{

  hhcd_USB_OTG_HS.Instance = USB_OTG_HS;
  hhcd_USB_OTG_HS.Init.Host_channels = 12;
  hhcd_USB_OTG_HS.Init.speed = HCD_SPEED_FULL;
  hhcd_USB_OTG_HS.Init.dma_enable = DISABLE;
  hhcd_USB_OTG_HS.Init.phy_itface = USB_OTG_EMBEDDED_PHY;
  hhcd_USB_OTG_HS.Init.Sof_enable = DISABLE;
  hhcd_USB_OTG_HS.Init.low_power_enable = DISABLE;
  hhcd_USB_OTG_HS.Init.vbus_sensing_enable = DISABLE;
  hhcd_USB_OTG_HS.Init.use_external_vbus = DISABLE;
  if( HAL_HCD_Init( &hhcd_USB_OTG_HS ) != HAL_OK )
  {
    _Error_Handler( __FILE__, __LINE__ );
  }

}

/**
  * Enable DMA controller clock
  */
static void
MX_DMA_Init( void )
{
  // DMA controller clock enable
  __HAL_RCC_DMA1_CLK_ENABLE(  );
  __HAL_RCC_DMA2_CLK_ENABLE(  );

  /* DMA interrupt init */
  /* DMA1_Stream4_IRQn interrupt configuration */
  HAL_NVIC_SetPriority( DMA1_Stream4_IRQn, 1, 0 );
  HAL_NVIC_EnableIRQ( DMA1_Stream4_IRQn );
  /* DMA1_Stream5_IRQn interrupt configuration */
  HAL_NVIC_SetPriority( DMA1_Stream5_IRQn, 1, 0 );
  HAL_NVIC_EnableIRQ( DMA1_Stream5_IRQn );
  /* DMA2_Stream0_IRQn interrupt configuration */
  HAL_NVIC_SetPriority( DMA2_Stream0_IRQn, 2, 0 );
  HAL_NVIC_EnableIRQ( DMA2_Stream0_IRQn );
  /* DMA2_Stream3_IRQn interrupt configuration */
  HAL_NVIC_SetPriority( DMA2_Stream3_IRQn, 2, 0 );
  HAL_NVIC_EnableIRQ( DMA2_Stream3_IRQn );

}

/** Configure pins as
	* Analog
	* Input
	* Output
	* EVENT_OUT
	* EXTI
	* Free pins are configured automatically as Analog (this feature is enabled through
	* the Code Generation settings)
*/
static void
MX_GPIO_Init( void )
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE(  );
  __HAL_RCC_GPIOH_CLK_ENABLE(  );
  __HAL_RCC_GPIOA_CLK_ENABLE(  );
  __HAL_RCC_GPIOB_CLK_ENABLE(  );
  __HAL_RCC_GPIOD_CLK_ENABLE(  );

  /* Configure GPIO pin Output Level */
  HAL_GPIO_WritePin( GPIOC, LED3_Pin | LED2_Pin | LED1_Pin | SoftMute1_Pin
      | SoftMute2_Pin | LED4_Pin, GPIO_PIN_RESET );

  /* Configure GPIO pin Output Level */
  HAL_GPIO_WritePin( GPIOA,
      LCD_DB4_Pin | LCD_DB5_Pin | LCD_DB6_Pin | LCD_DB7_Pin | LCD_RS_Pin |
      LCD_E_Pin, GPIO_PIN_RESET );

  /* Configure GPIO pins : LED3_Pin LED2_Pin LED1_Pin SoftMute1_Pin SoftMute2_Pin LED4_Pin */
  GPIO_InitStruct.Pin = LED3_Pin | LED2_Pin | LED1_Pin | SoftMute1_Pin
      | SoftMute2_Pin | LED4_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init( GPIOC, &GPIO_InitStruct );

  /* Configure GPIO pins : PC2 PC6 PC7 */
  GPIO_InitStruct.Pin = GPIO_PIN_2 | GPIO_PIN_6 | GPIO_PIN_7;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init( GPIOC, &GPIO_InitStruct );

  /* Configure GPIO pins : LCD_DB4_Pin LCD_DB5_Pin LCD_DB6_Pin LCD_DB7_Pin LCD_RS_Pin LCD_E_Pin */
  GPIO_InitStruct.Pin = LCD_DB4_Pin | LCD_DB5_Pin | LCD_DB6_Pin | LCD_DB7_Pin
      | LCD_RS_Pin | LCD_E_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init( GPIOA, &GPIO_InitStruct );

  /* Configure GPIO pins : PA7 PA8 PA9 PA10 */
  GPIO_InitStruct.Pin = GPIO_PIN_7 | GPIO_PIN_8 | GPIO_PIN_9 | GPIO_PIN_10;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init( GPIOA, &GPIO_InitStruct );

  /* Configure GPIO pin : Srf_Int_Pin */
  GPIO_InitStruct.Pin = Srf_Int_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init( Srf_Int_GPIO_Port, &GPIO_InitStruct );

  /* Configure GPIO pin : PB1 */
  GPIO_InitStruct.Pin = GPIO_PIN_1;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init( GPIOB, &GPIO_InitStruct );

  /* Configure GPIO pin : Boot1_Pin */
  GPIO_InitStruct.Pin = Boot1_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init( Boot1_GPIO_Port, &GPIO_InitStruct );

  /* Configure GPIO pin : Marker_Pin */
  GPIO_InitStruct.Pin = Marker_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init( Marker_GPIO_Port, &GPIO_InitStruct );

  /* EXTI interrupt init */
  HAL_NVIC_SetPriority( EXTI0_IRQn, 3, 0 );
  HAL_NVIC_EnableIRQ( EXTI0_IRQn );

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  file: The file name as string.
  * @param  line: The line in file as a number.
  * @retval None
  */
void
_Error_Handler( char *file, int line )
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while( 1 )
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef	USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *	    where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void
assert_failed( uint8_t * file, uint32_t line )
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number, printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif				/* USE_FULL_ASSERT */
